USE msdb;
GO

SET NOCOUNT ON;

DECLARE @foo TABLE
(
	-- move the PRIMARY KEY to the name column to
	-- see how it affects the various approaches
	id INT IDENTITY(1,1) PRIMARY KEY, 
	name NVARCHAR(257)
);

INSERT @foo 
	SELECT DISTINCT n = OBJECT_SCHEMA_NAME([object_id]) + '.' + name 
	FROM sys.objects AS so
	ORDER BY n;

DECLARE 
	@s		NVARCHAR(MAX), 
	@n		NVARCHAR(257), 
	@i		INT = 1,
	@runs	INT = 1000,
	@loop	INT = 0,
	@count  INT = (SELECT COUNT(*) FROM @foo),
	@now	DATETIME;

-- default cursor
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'Default cursor starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;

	DECLARE c CURSOR 
		FOR SELECT name FROM @foo;
		
	OPEN c;
	
	FETCH NEXT FROM c INTO @n;
	
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @s = COALESCE(@s + ',' + @n, @n);
		
		FETCH NEXT FROM c INTO @n;		
	END

	CLOSE c;
	DEALLOCATE c;

	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);

-- optimized cursor
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'Optimized cursor starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;

	DECLARE c CURSOR LOCAL STATIC FORWARD_ONLY READ_ONLY
		FOR SELECT name FROM @foo;
		
	OPEN c;
	
	FETCH NEXT FROM c INTO @n;
	
	WHILE @@FETCH_STATUS = 0
	BEGIN
		SET @s = COALESCE(@s + ',' + @n, @n);
		
		FETCH NEXT FROM c INTO @n;		
	END

	CLOSE c;
	DEALLOCATE c;

	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);

-- while loop
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'While loop starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;
	
	SET @loop = 0;
	
	WHILE @loop < @count
	BEGIN
		SELECT TOP 1 @s = COALESCE(@s + ',' + name, name)
			FROM @foo
			WHERE id > @loop
			ORDER BY id;
		
		SET @loop += 1;
	END	

	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);

-- set-based cursor
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'Set-based cursor starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;

	SELECT @s = COALESCE(@s + ',' + name, name)
	FROM @foo
	ORDER BY name;
	
	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);

-- for xml path
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'For xml path starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;

	WITH x(x) AS
	(
		SELECT name AS [data()] 
			FROM @foo 
			ORDER BY name
			FOR XML PATH('')
	)
	SELECT @s = REPLACE(x, ' ', ',')
		FROM x;

	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);

-- for xml path, type
SELECT @now = CURRENT_TIMESTAMP;
SELECT 'For xml path, type starting...', @now, 0;

SET @i = 1;

WHILE @i <= @runs
BEGIN
	SET @s = NULL;
	
	SELECT @s = 
	(
		SELECT name + ','
			FROM @foo
			ORDER BY name
			FOR XML PATH(''), TYPE
	).value('.[1]', 'NVARCHAR(MAX)');

	SELECT @s = LEFT(@s, LEN(@s)-1);
	
	SET @i += 1;
END

SELECT '...finished.', CURRENT_TIMESTAMP, DATEDIFF(MILLISECOND, @now, CURRENT_TIMESTAMP);